@@ -22,11 +22,11 @@ fi
 su - "$GFDOCKER_PRIMARY_USER" -c " \
   cd ~/gfarm \
     && (test -f Makefile && make distclean || true) \
-    && eval \"${CFLAGS_ARGS}\" ./configure --with-globus --enable-xmlattr \
+    && eval \"${CFLAGS_ARGS}\" ./configure --with-globus=/usr --enable-xmlattr \
     && make -j '${GFDOCKER_NUM_JOBS}' \
 " \
   && cd "/home/${GFDOCKER_PRIMARY_USER}/gfarm" \
-  && make install || exit 1
+  && make -j install || exit 1
 
 su - "$GFDOCKER_PRIMARY_USER" -c " \
   cd ~/gfarm2fs \
@@ -35,4 +35,4 @@ su - "$GFDOCKER_PRIMARY_USER" -c " \
     && make -j '${GFDOCKER_NUM_JOBS}' \
 " \
   && cd "/home/${GFDOCKER_PRIMARY_USER}/gfarm2fs" \
-  && make install || exit 1
+  && make -j install || exit 1
