@@ -16,6 +16,7 @@ gfurl_convert_gfs_stat(struct gfs_stat *from_st, struct gfurl_stat *to_st)
 {
 	to_st->nlink = from_st->st_nlink;
 	to_st->mode = from_st->st_mode;
+	to_st->ino = from_st->st_ino;
 	to_st->size = from_st->st_size;
 	to_st->mtime = from_st->st_mtimespec;
 }
@@ -72,6 +73,16 @@ gfurl_gfarm_chmod(const char *path, int mode)
 	return (e);
 }
 
+static gfarm_error_t
+gfurl_gfarm_rename(const char *from, const char *to)
+{
+	gfarm_error_t e;
+
+	e = gfs_rename(from, to);
+	gfmsg_debug_e(e, "gfs_rename(%s, %s)", from, to);
+	return (e);
+}
+
 static gfarm_error_t
 gfurl_gfarm_mkdir(const char *path, int mode, int skip_existing)
 {
@@ -94,6 +105,16 @@ gfurl_gfarm_rmdir(const char *path)
 	return (e);
 }
 
+static gfarm_error_t
+gfurl_gfarm_unlink(const char *path)
+{
+	gfarm_error_t e;
+
+	e = gfs_unlink(path);
+	gfmsg_debug_e(e, "gfs_unlink(%s)", path);
+	return (e);
+}
+
 static gfarm_error_t
 gfurl_gfarm_readlink(const char *path, char **targetp)
 {
@@ -127,8 +148,10 @@ const struct gfurl_functions gfurl_func_gfarm = {
 	.exist = gfurl_gfarm_exist,
 	.lutimens = gfurl_gfarm_lutimens,
 	.chmod = gfurl_gfarm_chmod,
+	.rename = gfurl_gfarm_rename,
 	.mkdir = gfurl_gfarm_mkdir,
 	.rmdir = gfurl_gfarm_rmdir,
+	.unlink = gfurl_gfarm_unlink,
 	.readlink = gfurl_gfarm_readlink,
 	.symlink = gfurl_gfarm_symlink,
 };
