@@ -530,6 +530,9 @@ gfarm_authorize_sharedsecret(struct gfp_xdr *conn, int switch_to,
 		sprintf(aux, "%s@%s", global_username, hostname);
 		gflog_set_auxiliary_info(aux);
 
+		gflog_info(GFARM_MSG_1004525,
+		    "(%s@%s) %s: authorize_sharedsecret: trying setuid...",
+		    global_username, hostname, local_username);
 		/*
 		 * because the name returned by getlogin() is
 		 * an attribute of a session on 4.4BSD derived OSs,
@@ -673,7 +676,7 @@ gfarm_authorize(struct gfp_xdr *conn,
 			return (e);
 		}
 		if (error != GFARM_AUTH_ERROR_NO_ERROR) {
-			gflog_warning(GFARM_MSG_1003369,
+			gflog_notice(GFARM_MSG_1003369,
 			    "%s: incorrect auth-method request %d: %d",
 			    hostname, (int)method, (int)error);
 			return (GFARM_ERR_PROTOCOL);
