@@ -119,7 +119,7 @@ extern int gfarm_replicainfo_enabled;
 #define GFARM_METADB_REMOVE_SCAN_LOG_INTERVAL_DEFAULT	3600 /* 3600 seconds */
 #define GFARM_METADB_REMOVE_SCAN_INTERVAL_FACTOR_DEFAULT 5 /* 1/5 */
 #define GFARM_METADB_HEARTBEAT_INTERVAL_DEFAULT 180 /* 3 min */
-#define GFARM_METADB_FAILOVER_NOTIFY_DELAY_DEFAULT 5 /* 5 seconds */
+#define GFARM_METADB_FAILOVER_NOTIFY_DELAY_DEFAULT 0 /* 0 second (no delay) */
 #define GFARM_METADB_DBQ_SIZE_DEFAULT	65536
 #define GFARM_SYMLINK_LEVEL_MAX			20
 
@@ -230,7 +230,6 @@ gfarm_error_t gfm_client_config_get_vars_result(struct gfm_connection *,
 	int, void **);
 
 /* miscellaneous */
-extern int gfarm_network_receive_timeout;
 extern int gfarm_file_trace;
 
 void gfarm_config_set_filename(char *);
