@@ -44,6 +44,7 @@ struct gfarm_filesystem {
 
 struct gfarm_filesystem_hash_id {
 	const char *hostname;
+	int port;
 };
 
 #define staticp	(gfarm_ctxp->filesystem_static)
@@ -74,6 +75,15 @@ gfarm_filesystem_static_init(struct gfarm_context *ctxp)
 	return (GFARM_ERR_NO_ERROR);
 }
 
+static void
+gfarm_filesystem_m2fs_hashtab_free(void)
+{
+	if (staticp->ms2fs_hashtab != NULL) {
+		gfarm_hash_table_free(staticp->ms2fs_hashtab);
+		staticp->ms2fs_hashtab = NULL;
+	}
+}
+
 void
 gfarm_filesystem_static_term(struct gfarm_context *ctxp)
 {
@@ -83,8 +93,7 @@ gfarm_filesystem_static_term(struct gfarm_context *ctxp)
 	if (s == NULL)
 		return;
 
-	if (staticp->ms2fs_hashtab != NULL)
-		gfarm_hash_table_free(staticp->ms2fs_hashtab);
+	gfarm_filesystem_m2fs_hashtab_free();
 	for (;;) {
 		fs = staticp->filesystems.next;
 		if (fs == &staticp->filesystems)
@@ -104,7 +113,7 @@ gfarm_filesystem_hash_index(const void *key, int keylen)
 	const struct gfarm_filesystem_hash_id *id = key;
 	const char *hostname = id->hostname;
 
-	return (gfarm_hash_casefold(hostname, strlen(hostname)));
+	return (gfarm_hash_casefold(hostname, strlen(hostname)) + id->port * 3);
 }
 
 static int
@@ -114,7 +123,8 @@ gfarm_filesystem_hash_equal(const void *key1, int key1len,
 	const struct gfarm_filesystem_hash_id *id1 = key1;
 	const struct gfarm_filesystem_hash_id *id2 = key2;
 
-	return (strcasecmp(id1->hostname, id2->hostname) == 0);
+	return (strcasecmp(id1->hostname, id2->hostname) == 0 &&
+	    id1->port == id2->port);
 }
 
 gfarm_error_t
@@ -201,6 +211,7 @@ gfarm_filesystem_hash_enter(struct gfarm_filesystem *fs,
 	static const char diag[] = "gfarm_filesystem_hash_enter";
 
 	id.hostname = gfarm_metadb_server_get_name(ms);
+	id.port = gfarm_metadb_server_get_port(ms);
 	entry = gfarm_hash_enter(staticp->ms2fs_hashtab, &id, sizeof(id),
 	    sizeof(fs), &created);
 	if (entry == NULL) {
@@ -227,6 +238,7 @@ gfarm_filesystem_hash_purge(struct gfarm_filesystem *fs,
 	int r;
 
 	id.hostname = gfarm_metadb_server_get_name(ms);
+	id.port = gfarm_metadb_server_get_port(ms);
 	r = gfarm_hash_purge(staticp->ms2fs_hashtab, &id, sizeof(id));
 	assert(r);
 	(void)r;
@@ -265,6 +277,7 @@ gfarm_filesystem_get(const char *hostname, int port)
 	if (staticp->ms2fs_hashtab == NULL)
 		return (NULL);
 	id.hostname = (char *)hostname; /* UNCONST */
+	id.port = port;
 	entry = gfarm_hash_lookup(staticp->ms2fs_hashtab, &id, sizeof(id));
 	return (entry ?
 	    *(struct gfarm_filesystem **)gfarm_hash_entry_data(entry) : NULL);
@@ -277,9 +290,10 @@ gfarm_filesystem_get_by_connection(struct gfm_connection *gfm_server)
 	    gfm_client_port(gfm_server)));
 }
 
-gfarm_error_t
-gfarm_filesystem_set_metadb_server_list(struct gfarm_filesystem *fs,
-	struct gfarm_metadb_server **metadb_servers, int n)
+static gfarm_error_t
+gfarm_filesystem_update_metadb_server_list(struct gfarm_filesystem *fs,
+	struct gfarm_metadb_server **metadb_servers, int n,
+	int do_purge)
 {
 	gfarm_error_t e;
 	int i;
@@ -299,7 +313,8 @@ gfarm_filesystem_set_metadb_server_list(struct gfarm_filesystem *fs,
 
 	for (i = 0; i < fs->nservers; ++i) {
 		gfarm_metadb_server_set_is_removed(fs->servers[i], 1);
-		gfarm_filesystem_hash_purge(fs, fs->servers[i]);
+		if (do_purge)
+			gfarm_filesystem_hash_purge(fs, fs->servers[i]);
 	}
 	for (i = 0; i < n; ++i)
 		gfarm_metadb_server_set_is_removed(metadb_servers[i], 0);
@@ -329,6 +344,31 @@ gfarm_filesystem_set_metadb_server_list(struct gfarm_filesystem *fs,
 	return (GFARM_ERR_NO_ERROR);
 }
 
+gfarm_error_t
+gfarm_filesystem_set_metadb_server_list(struct gfarm_filesystem *fs,
+	struct gfarm_metadb_server **metadb_servers, int n)
+{
+	return (gfarm_filesystem_update_metadb_server_list(
+	    fs, metadb_servers, n, 1));
+}
+
+gfarm_error_t
+gfarm_filesystem_replace_metadb_server_list(struct gfarm_filesystem *fs,
+	struct gfarm_metadb_server **metadb_servers, int n)
+{
+	/*
+	 * we cannot call gfarm_filesystem_hash_purge() by each fs->servers[i],
+	 * because gfarm_metadb_server_get_port(fs->servers[i]) may be
+	 * already broken by "gfmdhost -m -p <port>",
+	 * when this is called from mdhost_updated() in gfmd.
+	 */
+	gfarm_filesystem_m2fs_hashtab_free();
+
+	/* do_purge == false due to the reason above */
+	return (gfarm_filesystem_update_metadb_server_list(
+	    fs, metadb_servers, n, 0));
+}
+
 gfarm_error_t
 gfarm_filesystem_add(const char *hostname, int port,
 	struct gfarm_filesystem **fsp)
