@@ -69,28 +69,6 @@
 
 #define XAUTH_NEXTRACT_MAXLEN	512
 
-#ifndef __KERNEL__
-#define GFS_CLIENT_MUTEX		pthread_mutex_t client_mutex;
-#define GFS_CLIENT_MUTEX_INIT(s)	do { \
-	gfarm_mutex_init(&(s)->client_mutex, __func__, "client_mutex"); \
-} while (/*CONSTCOND*/0)
-#define GFS_CLIENT_MUTEX_DESTROY(s)	do { \
-	gfarm_mutex_destroy(&(s)->client_mutex, __func__, "client_mutex"); \
-} while (/*CONSTCOND*/0)
-#define GFS_CLIENT_MUTEX_LOCK(s)	do { \
-	gfarm_mutex_lock(&(s)->client_mutex, __func__, "client_mutex"); \
-} while (/*CONSTCOND*/0)
-#define GFS_CLIENT_MUTEX_UNLOCK(s)	do { \
-	gfarm_mutex_unlock(&(s)->client_mutex, __func__, "client_mutex"); \
-} while (/*CONSTCOND*/0)
-#else /* __KERNEL__ */
-#define GFS_CLIENT_MUTEX
-#define GFS_CLIENT_MUTEX_INIT(s)
-#define GFS_CLIENT_MUTEX_DESTROY(s)
-#define GFS_CLIENT_MUTEX_LOCK(s)
-#define GFS_CLIENT_MUTEX_UNLOCK(s)
-#endif /* __KERNEL__ */
-
 struct gfs_connection {
 	struct gfp_cached_connection *cache_entry;
 
@@ -118,12 +96,6 @@ struct gfs_connection {
 struct gfs_client_static {
 	struct gfp_conn_cache server_cache;
 	gfarm_error_t (*hook_for_connection_error)(struct gfs_connection *);
-
-	/* gfs_client_sockaddr_is_local() */
-	int self_ip_asked;
-	int self_ip_count;
-	struct in_addr *self_ip_list;
-	GFS_CLIENT_MUTEX;
 };
 
 #define SERVER_HASHTAB_SIZE	3079	/* prime number */
@@ -149,11 +121,6 @@ gfs_client_static_init(struct gfarm_context *ctxp)
 		SERVER_HASHTAB_SIZE,
 		&ctxp->gfsd_connection_cache);
 	s->hook_for_connection_error = NULL;
-	s->self_ip_asked = 0;
-	s->self_ip_count = 0;
-	s->self_ip_list = NULL;
-
-	GFS_CLIENT_MUTEX_INIT(s);
 
 	ctxp->gfs_client_static = s;
 	return (GFARM_ERR_NO_ERROR);
@@ -167,9 +134,7 @@ gfs_client_static_term(struct gfarm_context *ctxp)
 	if (s == NULL)
 		return;
 
-	GFS_CLIENT_MUTEX_DESTROY(s);
 	gfp_conn_cache_term(&s->server_cache);
-	free(staticp->self_ip_list);
 	free(s);
 }
 
@@ -181,9 +146,7 @@ void
 gfs_client_add_hook_for_connection_error(
 	gfarm_error_t (*hook)(struct gfs_connection *))
 {
-	GFS_CLIENT_MUTEX_LOCK(staticp);
 	staticp->hook_for_connection_error = hook;
-	GFS_CLIENT_MUTEX_UNLOCK(staticp);
 }
 
 static void
@@ -264,56 +227,61 @@ gfs_client_connection_set_failover_count(
 void
 gfs_client_purge_from_cache(struct gfs_connection *gfs_server)
 {
-	GFS_CLIENT_MUTEX_LOCK(staticp);
 	gfp_cached_connection_purge_from_cache(&staticp->server_cache,
 	    gfs_server->cache_entry);
-	GFS_CLIENT_MUTEX_UNLOCK(staticp);
 }
 
 void
 gfs_client_connection_gc(void)
 {
-	GFS_CLIENT_MUTEX_LOCK(staticp);
 	gfp_cached_connection_gc_all(&staticp->server_cache);
-	GFS_CLIENT_MUTEX_UNLOCK(staticp);
 }
 
-static int
-sockaddr_is_local_internal(struct sockaddr *peer_addr)
-{
-	struct sockaddr_in *peer_in;
-	int i;
+/*
+ * gfs_client_sockaddr_is_local()
+ *
+ * NOTE: kernel version won't need self_ip_count and self_ip_list
+ */
+#ifdef __KERNEL__
 
-	if (!gfarm_ctxp->direct_local_access)
-		return (0);
-	if (!staticp->self_ip_asked) {
-		staticp->self_ip_asked = 1;
-		if (gfarm_get_ip_addresses(&staticp->self_ip_count,
-		    &staticp->self_ip_list) != GFARM_ERR_NO_ERROR) {
-			/* self_ip_count remains 0 */
-			return (0);
-		}
-	}
-	if (peer_addr->sa_family != AF_INET)
-		return (0);
-	peer_in = (struct sockaddr_in *)peer_addr;
-	/* XXX if there are lots of IP address on this host, this is slow */
-	for (i = 0; i < staticp->self_ip_count; i++) {
-		if (peer_in->sin_addr.s_addr == staticp->self_ip_list[i].s_addr)
-			return (1);
-	}
-	return (0);
+/* kernel version won't need self_ip_count and self_ip_list */
+
+#else /* !__KERNEL__ */
+static int self_ip_count = 0;
+static struct in_addr *self_ip_list = NULL;
+
+static void
+self_ip_init(void)
+{
+	if (gfarm_get_ip_addresses(&self_ip_count, &self_ip_list) !=
+	    GFARM_ERR_NO_ERROR)
+		/* self_ip_count remains 0 */
+		return;
 }
 
 int
 gfs_client_sockaddr_is_local(struct sockaddr *peer_addr)
 {
-	int rv;
-	GFS_CLIENT_MUTEX_LOCK(staticp);
-	rv = sockaddr_is_local_internal(peer_addr);
-	GFS_CLIENT_MUTEX_UNLOCK(staticp);
-	return (rv);
+	struct sockaddr_in *peer_in;
+	int i;
+	static pthread_once_t self_ip_asked = PTHREAD_ONCE_INIT;
+
+	if (!gfarm_ctxp->direct_local_access)
+		return (0);
+	if (peer_addr->sa_family != AF_INET)
+		return (0);
+
+	pthread_once(&self_ip_asked, self_ip_init);
+
+	peer_in = (struct sockaddr_in *)peer_addr;
+	/* XXX if there are lots of IP address on this host, this is slow */
+	for (i = 0; i < self_ip_count; i++) {
+		if (peer_in->sin_addr.s_addr == self_ip_list[i].s_addr)
+			return (1);
+	}
+	return (0);
 }
+#endif /* !__KERNEL__ */
 
 #ifndef __KERNEL__	/* gfs_client_connect_xxx :: in user mode */
 static gfarm_error_t
@@ -449,7 +417,7 @@ gfs_client_connection_alloc(const char *canonical_hostname,
 #ifdef __GNUC__ /* workaround gcc warning: may be used uninitialized */
 	connection_in_progress = 0;
 #endif
-	if (sockaddr_is_local_internal(peer_addr)) {
+	if (gfs_client_sockaddr_is_local(peer_addr)) {
 		e = gfs_client_connect_unix(peer_addr, &sock);
 		if (e != GFARM_ERR_NO_ERROR) {
 			gflog_debug(GFARM_MSG_1001177,
@@ -693,24 +661,22 @@ gfs_client_connection_dispose(void *connection_data)
 void
 gfs_client_connection_free(struct gfs_connection *gfs_server)
 {
-	GFS_CLIENT_MUTEX_LOCK(staticp);
 	gfp_cached_or_uncached_connection_free(&staticp->server_cache,
 	    gfs_server->cache_entry);
-	GFS_CLIENT_MUTEX_UNLOCK(staticp);
 }
 
 void
 gfs_client_terminate(void)
 {
-	GFS_CLIENT_MUTEX_LOCK(staticp);
 	gfp_cached_connection_terminate(&staticp->server_cache);
-	GFS_CLIENT_MUTEX_UNLOCK(staticp);
 }
+
 void
 gfs_client_connection_unlock(struct gfs_connection *gfs_server)
 {
 	gfp_connection_unlock(gfs_server->cache_entry);
 }
+
 void
 gfs_client_connection_lock(struct gfs_connection *gfs_server)
 {
@@ -729,11 +695,6 @@ gfs_client_connection_acquire(const char *canonical_hostname, const char *user,
 	struct gfp_cached_connection *cache_entry;
 	int created;
 
-	GFS_CLIENT_MUTEX_LOCK(staticp);
-
-#ifdef __KERNEL__
-retry:
-#endif
 	e = gfp_cached_connection_acquire(&staticp->server_cache,
 	    canonical_hostname,
 	    ntohs(((struct sockaddr_in *)peer_addr)->sin_port),
@@ -743,36 +704,26 @@ retry:
 			"acquirement of cached connection (%s) failed: %s",
 			canonical_hostname,
 			gfarm_error_string(e));
-		GFS_CLIENT_MUTEX_UNLOCK(staticp);
 		return (e);
 	}
 	if (!created) {
 		*gfs_serverp = gfp_cached_connection_get_data(cache_entry);
-#ifdef __KERNEL__	/* workaround for race condition in MT */
-		if (!*gfs_serverp) {
-			gflog_warning(GFARM_MSG_1003895,
-				"gfs_client_connection_acquire:"
-				"%s not connected", canonical_hostname);
-			gfp_cached_or_uncached_connection_free(
-				&staticp->server_cache, cache_entry);
-			gfarm_nanosleep(10 * 1000 * 1000);
-			goto retry;
-		}
-#endif /* __KERNEL__ */
-		GFS_CLIENT_MUTEX_UNLOCK(staticp);
 		return (GFARM_ERR_NO_ERROR);
 	}
 	e = gfs_client_connection_alloc_and_auth(canonical_hostname, user,
 	    peer_addr, cache_entry, gfs_serverp, NULL, 0);
-	if (e != GFARM_ERR_NO_ERROR) {
-		gfp_cached_connection_purge_from_cache(&staticp->server_cache,
-		    cache_entry);
-		gfp_uncached_connection_dispose(cache_entry);
+	if (e == GFARM_ERR_NO_ERROR) {
+		gfp_cached_connection_initialization_succeeded(
+		    &staticp->server_cache, cache_entry);
+	} else {
+		gfp_cached_connection_initialization_aborted(
+		    &staticp->server_cache, cache_entry);
+		gfp_cached_or_uncached_connection_free(
+		    &staticp->server_cache, cache_entry);
 		gflog_debug(GFARM_MSG_1001185,
 			"allocation or authentication failed: %s",
 			gfarm_error_string(e));
 	}
-	GFS_CLIENT_MUTEX_UNLOCK(staticp);
 	return (e);
 }
 
@@ -788,11 +739,6 @@ gfs_client_connection_acquire_by_host(struct gfm_connection *gfm_server,
 	struct sockaddr peer_addr;
 	const char *user = gfm_client_username(gfm_server);
 
-	GFS_CLIENT_MUTEX_LOCK(staticp);
-
-#ifdef __KERNEL__
-retry:
-#endif /* __KERNEL__ */
 	/*
 	 * lookup gfs_server_cache first,
 	 * to eliminate hostname -> IP-address conversion in a cached case.
@@ -804,23 +750,10 @@ retry:
 			"acquirement of cached connection (%s) failed: %s",
 			canonical_hostname,
 			gfarm_error_string(e));
-		GFS_CLIENT_MUTEX_UNLOCK(staticp);
 		return (e);
 	}
 	if (!created) {
 		*gfs_serverp = gfp_cached_connection_get_data(cache_entry);
-#ifdef __KERNEL__	/* workaround for race condition in MT */
-		if (!*gfs_serverp) {
-			gflog_warning(GFARM_MSG_1003896,
-				"gfs_client_connection_acquire_by_host:"
-				"%s not connected", canonical_hostname);
-			gfp_cached_or_uncached_connection_free(
-				&staticp->server_cache, cache_entry);
-			gfarm_nanosleep(10 * 1000 * 1000);
-			goto retry;
-		}
-#endif /* __KERNEL__ */
-		GFS_CLIENT_MUTEX_UNLOCK(staticp);
 		return (GFARM_ERR_NO_ERROR);
 	}
 	e = gfm_host_address_get(gfm_server, canonical_hostname, port,
@@ -830,15 +763,18 @@ retry:
 		    user, &peer_addr, cache_entry, gfs_serverp, source_ip,
 		    gfarm_filesystem_failover_count(
 			gfarm_filesystem_get_by_connection(gfm_server)));
-	if (e != GFARM_ERR_NO_ERROR) {
-		gfp_cached_connection_purge_from_cache(&staticp->server_cache,
-		    cache_entry);
-		gfp_uncached_connection_dispose(cache_entry);
+	if (e == GFARM_ERR_NO_ERROR) {
+		gfp_cached_connection_initialization_succeeded(
+		    &staticp->server_cache, cache_entry);
+	} else {
+		gfp_cached_connection_initialization_aborted(
+		    &staticp->server_cache, cache_entry);
+		gfp_cached_or_uncached_connection_free(
+		    &staticp->server_cache, cache_entry);
 		gflog_debug(GFARM_MSG_1001187,
 			"client authentication failed: %s",
 			gfarm_error_string(e));
 	}
-	GFS_CLIENT_MUTEX_UNLOCK(staticp);
 	return (e);
 }
 
@@ -1026,7 +962,10 @@ gfs_client_connect(const char *canonical_hostname, int port, const char *user,
 	}
 	e = gfs_client_connection_alloc_and_auth(canonical_hostname, user,
 	    peer_addr, cache_entry, gfs_serverp, NULL, 0);
-	if (e != GFARM_ERR_NO_ERROR) {
+	if (e == GFARM_ERR_NO_ERROR) {
+		gfp_uncached_connection_initialization_succeeded(cache_entry);
+	} else {
+		gfp_uncached_connection_initialization_aborted(cache_entry);
 		gfp_uncached_connection_dispose(cache_entry);
 		gflog_debug(GFARM_MSG_1001189,
 			"client authentication failed: %s",
@@ -1040,7 +979,7 @@ gfarm_error_t
 gfs_client_connection_enter_cache(struct gfs_connection *gfs_server)
 {
 	gfarm_error_t e;
-	GFS_CLIENT_MUTEX_LOCK(staticp);
+
 	if (gfs_client_connection_is_cached(gfs_server)) {
 		gflog_fatal(GFARM_MSG_1000068,
 		    "gfs_client_connection_enter_cache: "
@@ -1048,14 +987,13 @@ gfs_client_connection_enter_cache(struct gfs_connection *gfs_server)
 	}
 	e = gfp_uncached_connection_enter_cache(&staticp->server_cache,
 	    gfs_server->cache_entry);
-	GFS_CLIENT_MUTEX_UNLOCK(staticp);
 	return (e);
 }
 gfarm_error_t
 gfs_client_connection_enter_cache_tail(struct gfs_connection *gfs_server)
 {
 	gfarm_error_t e;
-	GFS_CLIENT_MUTEX_LOCK(staticp);
+
 	if (gfs_client_connection_is_cached(gfs_server)) {
 		gflog_fatal(GFARM_MSG_1000068,
 		    "gfs_client_connection_enter_cache_tail: "
@@ -1063,16 +1001,14 @@ gfs_client_connection_enter_cache_tail(struct gfs_connection *gfs_server)
 	}
 	e = gfp_uncached_connection_enter_cache_tail(&staticp->server_cache,
 	    gfs_server->cache_entry);
-	GFS_CLIENT_MUTEX_UNLOCK(staticp);
 	return (e);
 }
 int
 gfs_client_connection_cache_change(int cnt)
 {
 	gfarm_error_t e;
-	GFS_CLIENT_MUTEX_LOCK(staticp);
+
 	e = gfp_connection_cache_change(&staticp->server_cache, cnt);
-	GFS_CLIENT_MUTEX_UNLOCK(staticp);
 	return (e);
 }
 
@@ -1243,6 +1179,7 @@ gfs_client_connect_request_multiplexed(struct gfarm_eventqueue *q,
 	    gfarm_filesystem_failover_count(fs));
 #endif /* __KERNEL__ */
 	if (e != GFARM_ERR_NO_ERROR) {
+		gfp_uncached_connection_initialization_aborted(cache_entry);
 		gfp_uncached_connection_dispose(cache_entry);
 		free(state);
 		gflog_debug(GFARM_MSG_1001193,
@@ -1328,7 +1265,12 @@ gfs_client_connect_result_multiplexed(
 	if (state->writable != NULL)
 		gfarm_event_free(state->writable);
 	free(state);
-	if (e != GFARM_ERR_NO_ERROR) {
+	if (e == GFARM_ERR_NO_ERROR) {
+		gfp_uncached_connection_initialization_succeeded(
+		    gfs_server->cache_entry);
+	} else {
+		gfp_uncached_connection_initialization_aborted(
+		    gfs_server->cache_entry);
 		gfs_client_connection_dispose(gfs_server);
 		gflog_debug(GFARM_MSG_1001195,
 			"error in result of multiplexed client connect: %s",
@@ -1566,6 +1508,19 @@ gfs_client_vrpc(struct gfs_connection *gfs_server, int just, int do_timeout,
 	return (GFARM_ERR_NO_ERROR);
 }
 
+static gfarm_error_t
+gfs_client_rpc_wo_lock(struct gfs_connection *gfs_server, int just,
+	int command, const char *format, ...)
+{
+	gfarm_error_t e;
+	va_list ap;
+
+	va_start(ap, format);
+	e = gfs_client_vrpc(gfs_server, just, 1, command, format, &ap);
+	va_end(ap);
+	return (e);
+}
+
 gfarm_error_t
 gfs_client_rpc(struct gfs_connection *gfs_server, int just,
 	int command, const char *format, ...)
@@ -1603,8 +1558,8 @@ gfs_client_process_set(struct gfs_connection *gfs_server,
 	gfarm_error_t e;
 
 	gfs_client_connection_lock(gfs_server);
-	e = gfs_client_rpc(gfs_server, 0, GFS_PROTO_PROCESS_SET, "ibl/",
-	    type, size, key, pid);
+	e = gfs_client_rpc_wo_lock(gfs_server, 0,
+	    GFS_PROTO_PROCESS_SET, "ibl/", type, size, key, pid);
 	if (e == GFARM_ERR_NO_ERROR)
 		gfs_server->pid = pid;
 	else
@@ -1622,7 +1577,7 @@ gfs_client_process_reset(struct gfs_connection *gfs_server,
 	gfarm_error_t e;
 
 	gfs_client_connection_lock(gfs_server);
-	e = gfs_client_rpc(gfs_server, 0, GFS_PROTO_PROCESS_RESET,
+	e = gfs_client_rpc_wo_lock(gfs_server, 0, GFS_PROTO_PROCESS_RESET,
 		"ibli/", type, size, key, pid, gfs_server->failover_count);
 	if (e == GFARM_ERR_NO_ERROR)
 		gfs_server->pid = pid;
@@ -1642,7 +1597,7 @@ gfs_client_open(struct gfs_connection *gfs_server, gfarm_int32_t fd)
 	gfarm_error_t e;
 
 	gfs_client_connection_lock(gfs_server);
-	e = gfs_client_rpc(gfs_server, 0, GFS_PROTO_OPEN, "i/", fd);
+	e = gfs_client_rpc_wo_lock(gfs_server, 0, GFS_PROTO_OPEN, "i/", fd);
 	if (e == GFARM_ERR_NO_ERROR)
 		++gfs_server->opened;
 	else
@@ -1670,7 +1625,8 @@ gfs_client_open_local(struct gfs_connection *gfs_server, gfarm_int32_t fd,
 
 	/* we have to set `just' flag here */
 	gfs_client_connection_lock(gfs_server);
-	e = gfs_client_rpc(gfs_server, 1, GFS_PROTO_OPEN_LOCAL, "i/", fd);
+	e = gfs_client_rpc_wo_lock(
+	    gfs_server, 1, GFS_PROTO_OPEN_LOCAL, "i/", fd);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gfs_client_connection_unlock(gfs_server);
 		gflog_debug(GFARM_MSG_1001205,
@@ -1719,7 +1675,7 @@ gfs_client_close(struct gfs_connection *gfs_server, gfarm_int32_t fd)
 
 	/* locked */
 	gfs_client_connection_lock(gfs_server);
-	e = gfs_client_rpc(gfs_server, 0, GFS_PROTO_CLOSE, "i/", fd);
+	e = gfs_client_rpc_wo_lock(gfs_server, 0, GFS_PROTO_CLOSE, "i/", fd);
 	if (e == GFARM_ERR_NO_ERROR)
 		--gfs_server->opened;
 	else
@@ -1738,7 +1694,7 @@ gfs_client_close_write(struct gfs_connection *gfs_server,
 
 	/* locked */
 	gfs_client_connection_lock(gfs_server);
-	e = gfs_client_rpc(gfs_server, 0, GFS_PROTO_CLOSE_WRITE, "ii/",
+	e = gfs_client_rpc_wo_lock(gfs_server, 0, GFS_PROTO_CLOSE_WRITE, "ii/",
 	    fd, flags);
 	if (e == GFARM_ERR_NO_ERROR)
 		--gfs_server->opened;
@@ -2617,8 +2573,10 @@ gfs_client_sendfile(struct gfs_connection *gfs_server,
 	gfarm_int32_t src_err = GFARM_ERR_NO_ERROR;
 	gfarm_off_t written = 0;
 
-	if ((e = gfs_client_rpc(gfs_server, 0, GFS_PROTO_BULKWRITE, "il/",
-	    remote_w_fd, w_off)) != GFARM_ERR_NO_ERROR) {
+	gfs_client_connection_lock(gfs_server);
+	if ((e = gfs_client_rpc_wo_lock(gfs_server, 0,
+	    GFS_PROTO_BULKWRITE, "il/", remote_w_fd, w_off))
+	    != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1003908,
 		    "gfs_client_sendfile: GFS_PROTO_BULKWRITE(%d, %lld): %s",
 		    remote_w_fd, (long long)w_off, gfarm_error_string(e));
@@ -2636,6 +2594,7 @@ gfs_client_sendfile(struct gfs_connection *gfs_server,
 				e = e2;
 		}
 	}
+	gfs_client_connection_unlock(gfs_server);
 	if (sentp != NULL)
 		*sentp = written;
 	return (e != GFARM_ERR_NO_ERROR ? e : src_err);
@@ -2654,8 +2613,10 @@ gfs_client_recvfile(struct gfs_connection *gfs_server,
 	gfarm_off_t written = 0;
 	int eof;
 
-	if ((e = gfs_client_rpc(gfs_server, 0, GFS_PROTO_BULKREAD, "ill/",
-	    remote_r_fd, len, r_off)) != GFARM_ERR_NO_ERROR) {
+	gfs_client_connection_lock(gfs_server);
+	if ((e = gfs_client_rpc_wo_lock(gfs_server, 0,
+	    GFS_PROTO_BULKREAD, "ill/", remote_r_fd, len, r_off))
+	    != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1003909,
 		    "gfs_client_recvfile: "
 		    "GFS_PROTO_BULKREAD(%d, %lld, %lld): %s",
@@ -2685,6 +2646,7 @@ gfs_client_recvfile(struct gfs_connection *gfs_server,
 			e = e2 != GFARM_ERR_NO_ERROR ? e2 :
 			    src_err != GFARM_ERR_NO_ERROR ? src_err : dst_err;
 	}
+	gfs_client_connection_unlock(gfs_server);
 	if (recvp != NULL)
 		*recvp = written;
 	return (e);
@@ -2708,6 +2670,7 @@ gfs_client_replica_recv_common(struct gfs_connection *gfs_server,
 {
 	gfarm_error_t e, e_rpc;
 
+	gfs_client_connection_lock(gfs_server);
 
 	if (cksum_protocol) {
 		e = gfs_client_rpc_request(gfs_server,
@@ -2727,6 +2690,7 @@ gfs_client_replica_recv_common(struct gfs_connection *gfs_server,
 		gflog_debug(GFARM_MSG_1001218,
 			"gfs_request_client_rpc() failed: %s",
 			gfarm_error_string(e));
+		gfs_client_connection_unlock(gfs_server);
 		return (e);
 	}
 
@@ -2748,6 +2712,8 @@ gfs_client_replica_recv_common(struct gfs_connection *gfs_server,
 		if (e == GFARM_ERR_NO_ERROR)
 			e = e_rpc;
 	}
+	gfs_client_connection_unlock(gfs_server);
+
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1001219,
 		    "receiving client replica failed: %s",
