@@ -58,6 +58,8 @@ struct gfarm_gfs_pio_static {
 	unsigned long long getline_count, getc_count, putc_count;
 };
 
+static gfarm_error_t flush_internal(GFS_File gf);
+
 gfarm_error_t
 gfarm_gfs_pio_static_init(struct gfarm_context *ctxp)
 {
@@ -109,6 +111,31 @@ struct gfs_file_list {
 	pthread_mutex_t mutex;
 };
 
+static void
+gfs_pio_mutex_init(pthread_mutex_t *mutex, const char *where)
+{
+	/* recursive lock may not be necessary but just in case */
+	gfarm_mutex_recursive_init(mutex, where, "GFS_File");
+}
+
+static void
+gfs_pio_mutex_destroy(pthread_mutex_t *mutex, const char *where)
+{
+	gfarm_mutex_destroy(mutex, where, "GFS_File");
+}
+
+static void
+gfs_pio_mutex_lock(pthread_mutex_t *mutex, const char *where)
+{
+	gfarm_mutex_lock(mutex, where, "GFS_File");
+}
+
+static void
+gfs_pio_mutex_unlock(pthread_mutex_t *mutex, const char *where)
+{
+	gfarm_mutex_unlock(mutex, where, "GFS_File");
+}
+
 /*
  * GFARM_ERRMSG_GFS_PIO_IS_EOF is used as mark of EOF,
  * and shouldn't be returned to caller functions.
@@ -117,7 +144,12 @@ struct gfs_file_list {
 int
 gfs_pio_eof(GFS_File gf)
 {
-	return (gf->error == GFARM_ERRMSG_GFS_PIO_IS_EOF);
+	int rv;
+
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+	rv = (gf->error == GFARM_ERRMSG_GFS_PIO_IS_EOF);
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+	return (rv);
 }
 
 #define GFS_PIO_ERROR(gf) \
@@ -125,15 +157,28 @@ gfs_pio_eof(GFS_File gf)
 	 (gf)->error : GFARM_ERR_NO_ERROR)
 
 gfarm_error_t
-gfs_pio_error(GFS_File gf)
+gfs_pio_error_unlocked(GFS_File gf)
 {
 	return (GFS_PIO_ERROR(gf));
 }
 
+gfarm_error_t
+gfs_pio_error(GFS_File gf)
+{
+	gfarm_error_t e;
+
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+	e = gfs_pio_error_unlocked(gf);
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+	return (e);
+}
+
 void
 gfs_pio_clearerr(GFS_File gf)
 {
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
 	gf->error = GFARM_ERR_NO_ERROR;
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
 }
 
 static gfarm_error_t
@@ -149,7 +194,7 @@ gfs_pio_set_view_default(GFS_File gf)
 
 	if (gfs_pio_is_view_set(gf)) {
 		if ((gf->mode & GFS_FILE_MODE_WRITE) != 0)
-			e_save = gfs_pio_flush(gf);
+			e_save = flush_internal(gf);
 		e = (*gf->ops->view_close)(gf);
 		if (e_save == GFARM_ERR_NO_ERROR)
 			e_save = e;
@@ -335,7 +380,7 @@ gfs_file_alloc(struct gfm_connection *gfm_server, gfarm_int32_t fd, int flags,
 	struct gfs_pio_internal_cksum_info *cip, GFS_File *gfp)
 {
 	GFS_File gf;
-	char *buffer;
+	char *buffer = NULL;
 	struct gfs_file_list *gfl;
 
 	GFARM_MALLOC(gf);
@@ -363,9 +408,8 @@ gfs_file_alloc(struct gfm_connection *gfm_server, gfarm_int32_t fd, int flags,
 	if ((flags & GFARM_FILE_TRUNC) != 0)
 		gf->mode |= GFS_FILE_MODE_MODIFIED;
 
-	if ((flags & GFARM_FILE_UNBUFFERED) != 0) {
-		buffer = NULL;
-	} else {
+	if ((flags & GFARM_FILE_UNBUFFERED) == 0 &&
+	    gfarm_ctxp->client_file_bufsize > 0) {
 		GFARM_MALLOC_ARRAY(buffer, gfarm_ctxp->client_file_bufsize);
 		if (buffer == NULL) {
 			free(gf);
@@ -422,6 +466,7 @@ gfs_file_alloc(struct gfm_connection *gfm_server, gfarm_int32_t fd, int flags,
 	    gfm_server);
 #endif
 #endif /* __KERNEL__ */
+	gfs_pio_mutex_init(&gf->mutex, __func__);
 	gfs_pio_file_list_add(gfl, gf);
 
 	*gfp = gf;
@@ -431,11 +476,11 @@ gfs_file_alloc(struct gfm_connection *gfm_server, gfarm_int32_t fd, int flags,
 static void
 gfs_file_free(GFS_File gf)
 {
-	if (!(gf->open_flags & GFARM_FILE_UNBUFFERED))
-		free(gf->buffer);
+	free(gf->buffer);
 	free(gf->url);
 	free(gf->md.cksum_type);
 	/* do not touch gf->pi here */
+	gfs_pio_mutex_destroy(&gf->mutex, __func__);
 	free(gf);
 }
 
@@ -764,6 +809,8 @@ gfs_pio_close_getgen(GFS_File gf, gfarm_uint64_t *igenp)
 	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t1);
 	gfs_profile(gfarm_gettimerval(&t1));
 
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+
 	/*
 	 * no need to check and set the default file view here
 	 * because neither gfs_pio_flush nor view_close is not
@@ -776,7 +823,7 @@ gfs_pio_close_getgen(GFS_File gf, gfarm_uint64_t *igenp)
 
 		is_local = vc->ops == &gfs_pio_local_storage_ops;
 		if ((gf->mode & GFS_FILE_MODE_WRITE) != 0)
-			e_save = gfs_pio_flush(gf);
+			e_save = flush_internal(gf);
 
 		/* for client-side cksum calculation */
 		if ((gf->mode &
@@ -887,6 +934,8 @@ gfs_pio_close_getgen(GFS_File gf, gfarm_uint64_t *igenp)
 			e_save = e;
 	}
 
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+
 	gfm_client_connection_free(gf->gfm_server);
 	gfs_file_free(gf);
 
@@ -922,6 +971,13 @@ gfs_pio_purge(GFS_File gf)
 	else if ((gf)->error == GFARM_ERRMSG_GFS_PIO_IS_EOF) \
 		(gf)->error = GFARM_ERR_NO_ERROR; \
 }
+#define CHECK_WRITABLE_LOCKED(gf) { \
+	if (((gf)->mode & GFS_FILE_MODE_WRITE) == 0) {\
+		gfs_pio_mutex_unlock(&(gf)->mutex, __func__); \
+		return (gfarm_errno_to_error(EBADF)); \
+	} else if ((gf)->error == GFARM_ERRMSG_GFS_PIO_IS_EOF) \
+		(gf)->error = GFARM_ERR_NO_ERROR; \
+}
 /*
  * we check this against gf->open_flags rather than gf->mode,
  * because we may set GFARM_FILE_MODE_READ even if write-only case.
@@ -936,6 +992,20 @@ gfs_pio_purge(GFS_File gf)
 		return (EOF); \
 }
 
+#define CHECK_READABLE_LOCKED(gf) { \
+	if (((gf)->open_flags & GFARM_FILE_ACCMODE) == GFARM_FILE_WRONLY) { \
+		gfs_pio_mutex_unlock(&(gf)->mutex, __func__); \
+		return (gfarm_errno_to_error(EBADF)); \
+	} \
+}
+
+#define CHECK_READABLE_EOF_LOCKED(gf) { \
+	if (((gf)->open_flags & GFARM_FILE_ACCMODE) == GFARM_FILE_WRONLY) { \
+		gfs_pio_mutex_unlock(&(gf)->mutex, __func__); \
+		return (EOF); \
+	} \
+}
+
 static gfarm_error_t
 gfs_pio_fillbuf(GFS_File gf, size_t size)
 {
@@ -953,11 +1023,16 @@ gfs_pio_fillbuf(GFS_File gf, size_t size)
 		}
 		return (GFS_PIO_ERROR(gf));
 	}
+	/* UNBUFFERED case */
+	if (gf->buffer == NULL) {
+		gf->error = GFARM_ERR_NO_BUFFER_SPACE_AVAILABLE;
+		return (gf->error);
+	}
 	if (gf->p < gf->length)
 		return (GFARM_ERR_NO_ERROR);
 
 	if ((gf->mode & GFS_FILE_MODE_BUFFER_DIRTY) != 0) {
-		e = gfs_pio_flush(gf);
+		e = flush_internal(gf);
 		if (e != GFARM_ERR_NO_ERROR) {
 			gflog_debug(GFARM_MSG_1001301,
 				"gfs_pio_flush() failed: %s",
@@ -1003,6 +1078,7 @@ do_write(GFS_File gf, const char *buffer, size_t length,
 		*writtenp = 0;
 		return (GFARM_ERR_NO_ERROR);
 	}
+	assert(buffer);
 	if (gf->io_offset != gf->offset) {
 		gf->io_offset = gf->offset;
 	}
@@ -1029,8 +1105,8 @@ do_write(GFS_File gf, const char *buffer, size_t length,
 	return (e);
 }
 
-gfarm_error_t
-gfs_pio_flush(GFS_File gf)
+static gfarm_error_t
+flush_internal(GFS_File gf)
 {
 	gfarm_error_t e;
 	size_t written;
@@ -1061,7 +1137,18 @@ gfs_pio_flush(GFS_File gf)
 }
 
 gfarm_error_t
-gfs_pio_seek(GFS_File gf, gfarm_off_t offset, int whence, gfarm_off_t *resultp)
+gfs_pio_flush(GFS_File gf)
+{
+	gfarm_error_t e;
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+	e = flush_internal(gf);
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+	return (e);
+}
+
+static gfarm_error_t
+seek_internal(GFS_File gf, gfarm_off_t offset, int whence,
+	gfarm_off_t *resultp)
 {
 	gfarm_error_t e;
 	gfarm_off_t where;
@@ -1129,7 +1216,7 @@ gfs_pio_seek(GFS_File gf, gfarm_off_t offset, int whence, gfarm_off_t *resultp)
 	}
 
 	if (gf->mode & GFS_FILE_MODE_BUFFER_DIRTY) {
-		e = gfs_pio_flush(gf);
+		e = flush_internal(gf);
 		if (e != GFARM_ERR_NO_ERROR) {
 			gf->error = e;
 			gflog_debug(GFARM_MSG_1001310,
@@ -1153,6 +1240,16 @@ gfs_pio_seek(GFS_File gf, gfarm_off_t offset, int whence, gfarm_off_t *resultp)
 	return (e);
 }
 
+gfarm_error_t
+gfs_pio_seek(GFS_File gf, gfarm_off_t offset, int whence, gfarm_off_t *resultp)
+{
+	gfarm_error_t e;
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+	e = seek_internal(gf, offset, whence, resultp);
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+	return (e);
+}
+
 gfarm_error_t
 gfs_pio_truncate(GFS_File gf, gfarm_off_t length)
 {
@@ -1164,6 +1261,8 @@ gfs_pio_truncate(GFS_File gf, gfarm_off_t length)
 	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t1);
 	gfs_profile(gfarm_gettimerval(&t1));
 
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+
 	e = gfs_pio_check_view_default(gf);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1001311,
@@ -1172,10 +1271,10 @@ gfs_pio_truncate(GFS_File gf, gfarm_off_t length)
 		goto finish;
 	}
 
-	CHECK_WRITABLE(gf);
+	CHECK_WRITABLE_LOCKED(gf);
 
 	if (gf->mode & GFS_FILE_MODE_BUFFER_DIRTY) {
-		e = gfs_pio_flush(gf);
+		e = flush_internal(gf);
 		if (e != GFARM_ERR_NO_ERROR) {
 			gflog_debug(GFARM_MSG_1001312,
 				"gfs_pio_flush(): %s",
@@ -1194,6 +1293,7 @@ gfs_pio_truncate(GFS_File gf, gfarm_off_t length)
 	if (e != GFARM_ERR_NO_ERROR)
 		gf->error = e;
 finish:
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
 	gfs_profile(gfarm_gettimerval(&t2));
 	gfs_profile(staticp->truncate_time += gfarm_timerval_sub(&t2, &t1));
 	gfs_profile(staticp->truncate_count++);
@@ -1280,8 +1380,8 @@ gfs_pio_pwrite_unbuffer(GFS_File gf, const void *buffer, int size,
 	return (e);
 }
 
-gfarm_error_t
-gfs_pio_read(GFS_File gf, void *buffer, int size, int *np)
+static gfarm_error_t
+read_internal(GFS_File gf, void *buffer, int size, int *np)
 {
 	gfarm_error_t e;
 	char *p = buffer;
@@ -1303,11 +1403,16 @@ gfs_pio_read(GFS_File gf, void *buffer, int size, int *np)
 
 	CHECK_READABLE(gf);
 
-	if (!gf->buffer) {
+	/* UNBUFFERED case */
+	if (gf->buffer == NULL) {
 		gfarm_off_t result, offset = gf->offset + gf->p;
+
 		e = gfs_pio_pread_unbuffer(gf, buffer, size, offset, np);
-		if (e == GFARM_ERR_NO_ERROR)
-			gfs_pio_seek(gf, offset + *np, GFARM_SEEK_SET, &result);
+		if (e == GFARM_ERR_NO_ERROR) {
+			seek_internal(gf,
+			    offset + *np, GFARM_SEEK_SET, &result);
+			n = *np;
+		}
 		goto finish;
 	}
 	if (size >= gf->bufsize + (gf->length - gf->p)) {
@@ -1320,7 +1425,7 @@ gfs_pio_read(GFS_File gf, void *buffer, int size, int *np)
 			gf->p += length;
 		}
 		if ((gf->mode & GFS_FILE_MODE_BUFFER_DIRTY) != 0)
-			e = gfs_pio_flush(gf); /* this does purge too */
+			e = flush_internal(gf); /* this does purge too */
 		else
 			e = gfs_pio_purge(gf);
 		if (e == GFARM_ERR_NO_ERROR) {
@@ -1375,6 +1480,17 @@ gfs_pio_read(GFS_File gf, void *buffer, int size, int *np)
 
 	return (e);
 }
+
+gfarm_error_t
+gfs_pio_read(GFS_File gf, void *buffer, int size, int *np)
+{
+	gfarm_error_t e;
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+	e = read_internal(gf, buffer, size, np);
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+	return (e);
+}
+
 gfarm_error_t
 gfs_pio_pread_page(GFS_File gf, gfarm_off_t off, int size, int force,
 		gfs_pageio_t cb, void *arg)
@@ -1387,22 +1503,25 @@ gfs_pio_pread_page(GFS_File gf, gfarm_off_t off, int size, int force,
 	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t1);
 	gfs_profile(gfarm_gettimerval(&t1));
 
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+
 	e = gfs_pio_check_view_default(gf);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1004616,
 			"Check view default for pio failed: %s",
 			gfarm_error_string(e));
+		gfs_pio_mutex_unlock(&gf->mutex, __func__);
 		return (e);
 	}
 
-	CHECK_READABLE(gf);
+	CHECK_READABLE_LOCKED(gf);
 
 	do {
 		if (second || off < gf->offset
 			|| off >= (gf->offset + gf->length)) {
 			if (!force)
 				break;
-			gfs_pio_flush(gf);
+			flush_internal(gf);
 			gfs_pio_purge(gf);
 			gf->offset = off;
 
@@ -1435,17 +1554,18 @@ gfs_pio_pread_page(GFS_File gf, gfarm_off_t off, int size, int force,
 
 	e = GFARM_ERR_NO_ERROR;
  finish:
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
 	gfs_profile(gfarm_gettimerval(&t2));
 	gfs_profile(staticp->read_time += gfarm_timerval_sub(&t2, &t1));
 
 	return (e);
 }
 
-gfarm_error_t
-gfs_pio_write(GFS_File gf, const void *buffer, int size, int *np)
+static gfarm_error_t
+write_internal(GFS_File gf, const void *buffer, int size, int *np)
 {
 	gfarm_error_t e;
-	size_t written;
+	size_t written = 0;
 	gfarm_timerval_t t1, t2;
 
 	GFARM_KERNEL_UNUSE2(t1, t2);
@@ -1462,12 +1582,16 @@ gfs_pio_write(GFS_File gf, const void *buffer, int size, int *np)
 
 	CHECK_WRITABLE(gf);
 
+	/* UNBUFFERED case */
 	if (gf->buffer == NULL) {
 		gfarm_off_t result, offset = gf->offset + gf->p;
 
 		e = gfs_pio_pwrite_unbuffer(gf, buffer, size, offset, np);
-		if (e == GFARM_ERR_NO_ERROR)
-			gfs_pio_seek(gf, offset + *np, GFARM_SEEK_SET, &result);
+		if (e == GFARM_ERR_NO_ERROR) {
+			seek_internal(gf,
+			    offset + *np, GFARM_SEEK_SET, &result);
+			written = *np;
+		}
 		goto finish;
 	}
 
@@ -1477,7 +1601,7 @@ gfs_pio_write(GFS_File gf, const void *buffer, int size, int *np)
 		 * by buffer.
 		 */
 		gf->length = gf->p;
-		e = gfs_pio_flush(gf); /* this does purge too */
+		e = flush_internal(gf); /* this does purge too */
 		if (e != GFARM_ERR_NO_ERROR) {
 			gflog_debug(GFARM_MSG_1001316,
 				"gfs_pio_flush() failed: %s",
@@ -1512,37 +1636,52 @@ gfs_pio_write(GFS_File gf, const void *buffer, int size, int *np)
 	gf->p += size;
 	if (gf->p > gf->length)
 		gf->length = gf->p;
-	*np = size;
+	*np = written = size;
 	e = GFARM_ERR_NO_ERROR;
 	if (gf->p >= gf->bufsize)
-		e = gfs_pio_flush(gf);
+		e = flush_internal(gf);
  finish:
 	gfs_profile(gfarm_gettimerval(&t2));
 	gfs_profile(staticp->write_time += gfarm_timerval_sub(&t2, &t1));
-	gfs_profile(staticp->write_size += size);
+	gfs_profile(staticp->write_size += written);
 	gfs_profile(staticp->write_count++);
 
 	return (e);
 }
 
+gfarm_error_t
+gfs_pio_write(GFS_File gf, const void *buffer, int size, int *np)
+{
+	gfarm_error_t e;
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+	e = write_internal(gf, buffer, size, np);
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+	return (e);
+}
+
 gfarm_error_t
 gfs_pio_pread(GFS_File gf, void *buffer, int size, gfarm_off_t offset, int *np)
 {
 	gfarm_error_t e;
 
-	if (!(e = gfs_pio_seek(gf, offset, GFARM_SEEK_SET, &offset)))
-		e = gfs_pio_read(gf, buffer, size, np);
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+	if (!(e = seek_internal(gf, offset, GFARM_SEEK_SET, &offset)))
+		e = read_internal(gf, buffer, size, np);
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
 
 	return (e);
 }
 
 gfarm_error_t
-gfs_pio_pwrite(GFS_File gf, void *buffer, int size, gfarm_off_t offset, int *np)
+gfs_pio_pwrite(GFS_File gf,
+		const void *buffer, int size, gfarm_off_t offset, int *np)
 {
 	gfarm_error_t e;
 
-	if (!(e = gfs_pio_seek(gf, offset, GFARM_SEEK_SET, &offset)))
-		e = gfs_pio_write(gf, buffer, size, np);
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+	if (!(e = seek_internal(gf, offset, GFARM_SEEK_SET, &offset)))
+		e = write_internal(gf, buffer, size, np);
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
 
 	return (e);
 }
@@ -1560,15 +1699,18 @@ gfs_pio_append(GFS_File gf, void *buffer, int size, int *np,
 	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t1);
 	gfs_profile(gfarm_gettimerval(&t1));
 
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+
 	e = gfs_pio_check_view_default(gf);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1003948,
 			"gfs_pio_check_view_default() failed: %s",
 			gfarm_error_string(e));
+		gfs_pio_mutex_unlock(&gf->mutex, __func__);
 		return (e);
 	}
 
-	CHECK_WRITABLE(gf);
+	CHECK_WRITABLE_LOCKED(gf);
 
 	do {
 		e = (*gf->ops->view_write)(gf,
@@ -1579,10 +1721,13 @@ gfs_pio_append(GFS_File gf, void *buffer, int size, int *np,
 		gflog_debug(GFARM_MSG_1003949,
 			"view_write() failed: %s",
 			gfarm_error_string(e));
+		gfs_pio_mutex_unlock(&gf->mutex, __func__);
 		return (e);
 	}
 	*np = length;
 
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+
 	gfs_profile(gfarm_gettimerval(&t2));
 	gfs_profile(staticp->write_time += gfarm_timerval_sub(&t2, &t1));
 
@@ -1627,7 +1772,7 @@ sync_internal(GFS_File gf, int operation, double *time, unsigned long long *ct)
 		    "gfs_pio_sync: %s", gfarm_error_string(e));
 		return (e);
 	}
-	e = gfs_pio_flush(gf);
+	e = flush_internal(gf);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1001318,
 			"gfs_pio_flush() failed: %s",
@@ -1656,19 +1801,29 @@ finish:
 gfarm_error_t
 gfs_pio_sync(GFS_File gf)
 {
-	return (sync_internal(gf, GFS_PROTO_FSYNC_WITH_METADATA,
-		    &staticp->sync_time, &staticp->sync_count));
+	gfarm_error_t e;
+
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+	e = sync_internal(gf, GFS_PROTO_FSYNC_WITH_METADATA,
+		    &staticp->sync_time, &staticp->sync_count);
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+	return (e);
 }
 
 gfarm_error_t
 gfs_pio_datasync(GFS_File gf)
 {
-	return (sync_internal(gf, GFS_PROTO_FSYNC_WITHOUT_METADATA,
-		    &staticp->datasync_time, &staticp->datasync_count));
+	gfarm_error_t e;
+
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+	e = sync_internal(gf, GFS_PROTO_FSYNC_WITHOUT_METADATA,
+		    &staticp->datasync_time, &staticp->datasync_count);
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+	return (e);
 }
 
-int
-gfs_pio_getc(GFS_File gf)
+static int
+getc_internal(GFS_File gf)
 {
 	gfarm_error_t e;
 	int c;
@@ -1705,6 +1860,7 @@ gfs_pio_getc(GFS_File gf)
 			goto finish;
 		}
 	}
+	assert(gf->buffer);
 	c = ((unsigned char *)gf->buffer)[gf->p++];
  finish:
 	gfs_profile(gfarm_gettimerval(&t2));
@@ -1713,10 +1869,21 @@ gfs_pio_getc(GFS_File gf)
 }
 
 int
-gfs_pio_ungetc(GFS_File gf, int c)
+gfs_pio_getc(GFS_File gf)
 {
-	gfarm_error_t e = gfs_pio_check_view_default(gf);
+	int c;
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+	c = getc_internal(gf);
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+	return (c);
+}
 
+static int
+ungetc_internal(GFS_File gf, int c)
+{
+	gfarm_error_t e;
+
+	e = gfs_pio_check_view_default(gf);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gf->error = e;
 		gflog_debug(GFARM_MSG_1001323,
@@ -1736,13 +1903,24 @@ gfs_pio_ungetc(GFS_File gf, int c)
 			return (EOF);
 		}
 		/* We do not mark this buffer dirty here. */
+		assert(gf->buffer);
 		gf->buffer[--gf->p] = c;
 	}
 	return (c);
 }
 
-gfarm_error_t
-gfs_pio_putc(GFS_File gf, int c)
+int
+gfs_pio_ungetc(GFS_File gf, int c)
+{
+	int c2;
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+	c2 = ungetc_internal(gf, c);
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+	return (c2);
+}
+
+static gfarm_error_t
+putc_internal(GFS_File gf, int c)
 {
 	gfarm_error_t e;
 	gfarm_timerval_t t1, t2;
@@ -1761,8 +1939,13 @@ gfs_pio_putc(GFS_File gf, int c)
 
 	CHECK_WRITABLE(gf);
 
+	/* UNBUFFERED case */
+	if (gf->buffer == NULL) {
+		gf->error = GFARM_ERR_NO_BUFFER_SPACE_AVAILABLE;
+		return (gf->error);
+	}
 	if (gf->p >= gf->bufsize) {
-		gfarm_error_t e = gfs_pio_flush(gf); /* this does purge too */
+		gfarm_error_t e = flush_internal(gf); /* this does purge too */
 
 		if (e != GFARM_ERR_NO_ERROR) {
 			gflog_debug(GFARM_MSG_1001326,
@@ -1779,19 +1962,30 @@ gfs_pio_putc(GFS_File gf, int c)
 	if (gf->p > gf->length)
 		gf->length = gf->p;
 	if (gf->p >= gf->bufsize)
-		e = gfs_pio_flush(gf);
+		e = flush_internal(gf);
  finish:
 	gfs_profile(gfarm_gettimerval(&t2));
 	gfs_profile(staticp->putc_time += gfarm_timerval_sub(&t2, &t1));
 	return (e);
 }
 
-/* mostly compatible with fgets(3) */
 gfarm_error_t
-gfs_pio_puts(GFS_File gf, const char *s)
+gfs_pio_putc(GFS_File gf, int c)
 {
-	gfarm_error_t e = gfs_pio_check_view_default(gf);
+	gfarm_error_t e;
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+	e = putc_internal(gf, c);
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+	return (e);
+}
 
+/* mostly compatible with fgets(3) */
+static gfarm_error_t
+puts_internal(GFS_File gf, const char *s)
+{
+	gfarm_error_t e;
+
+	e = gfs_pio_check_view_default(gf);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1001327,
 			"gfs_pio_check_view_default() failed: %s",
@@ -1802,7 +1996,7 @@ gfs_pio_puts(GFS_File gf, const char *s)
 	CHECK_WRITABLE(gf);
 
 	while (*s != '\0') {
-		gfarm_error_t e = gfs_pio_putc(gf, *(unsigned char *)s);
+		gfarm_error_t e = putc_internal(gf, *(unsigned char *)s);
 
 		if (e != GFARM_ERR_NO_ERROR) {
 			gflog_debug(GFARM_MSG_1001328,
@@ -1815,19 +2009,33 @@ gfs_pio_puts(GFS_File gf, const char *s)
 	return (GFARM_ERR_NO_ERROR);
 }
 
+gfarm_error_t
+gfs_pio_puts(GFS_File gf, const char *s)
+{
+	gfarm_error_t e;
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+	e = puts_internal(gf, s);
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+	return (e);
+}
+
 /* mostly compatible with fgets(3), but EOF check is done by *s == '\0' */
 gfarm_error_t
 gfs_pio_gets(GFS_File gf, char *s, size_t size)
 {
-	gfarm_error_t e = gfs_pio_check_view_default(gf);
+	gfarm_error_t e;
 	char *p = s;
 	int c;
 	gfarm_timerval_t t1, t2;
 
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+
+	e = gfs_pio_check_view_default(gf);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1001329,
 			"gfs_pio_check_view_default() failed: %s",
 			gfarm_error_string(e));
+		gfs_pio_mutex_unlock(&gf->mutex, __func__);
 		return (e);
 	}
 
@@ -1838,7 +2046,7 @@ gfs_pio_gets(GFS_File gf, char *s, size_t size)
 #ifdef __GNUC__ /* workaround gcc warning: unused variable */
 	c = EOF;
 #endif
-	CHECK_READABLE(gf);
+	CHECK_READABLE_LOCKED(gf);
 
 	if (size <= 1) {
 		gf->error = GFARM_ERR_INVALID_ARGUMENT;
@@ -1846,16 +2054,19 @@ gfs_pio_gets(GFS_File gf, char *s, size_t size)
 			"invalid argument, size (%d) <= 1: %s",
 			(int)size,
 			gfarm_error_string(gf->error));
+		gfs_pio_mutex_unlock(&gf->mutex, __func__);
 		return (gf->error);
 	}
 	--size; /* for '\0' */
-	for (; size > 0 && (c = gfs_pio_getc(gf)) != EOF; --size) {
+	for (; size > 0 && (c = getc_internal(gf)) != EOF; --size) {
 		*p++ = c;
 		if (c == '\n')
 			break;
 	}
 	*p++ = '\0';
 
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+
 	gfs_profile(gfarm_gettimerval(&t2));
 	/* XXX should introduce gfs_pio_gets_time??? */
 	gfs_profile(staticp->getline_time += gfarm_timerval_sub(&t2, &t1));
@@ -1866,15 +2077,19 @@ gfs_pio_gets(GFS_File gf, char *s, size_t size)
 gfarm_error_t
 gfs_pio_getline(GFS_File gf, char *s, size_t size, int *eofp)
 {
-	gfarm_error_t e = gfs_pio_check_view_default(gf);
+	gfarm_error_t e;
 	char *p = s;
 	int c;
 	gfarm_timerval_t t1, t2;
 
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+
+	e = gfs_pio_check_view_default(gf);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1001331,
 			"gfs_pio_check_view_default() failed: %s",
 			gfarm_error_string(e));
+		gfs_pio_mutex_unlock(&gf->mutex, __func__);
 		return (e);
 	}
 
@@ -1885,7 +2100,7 @@ gfs_pio_getline(GFS_File gf, char *s, size_t size, int *eofp)
 #ifdef __GNUC__ /* workaround gcc warning: unused variable */
 	c = EOF;
 #endif
-	CHECK_READABLE(gf);
+	CHECK_READABLE_LOCKED(gf);
 
 	if (size <= 1) {
 		gf->error = GFARM_ERR_INVALID_ARGUMENT;
@@ -1893,10 +2108,11 @@ gfs_pio_getline(GFS_File gf, char *s, size_t size, int *eofp)
 			"invalid argument, size(%d) <= 1: %s",
 			(int)size,
 			gfarm_error_string(gf->error));
+		gfs_pio_mutex_unlock(&gf->mutex, __func__);
 		return (gf->error);
 	}
 	--size; /* for '\0' */
-	for (; size > 0 && (c = gfs_pio_getc(gf)) != EOF; --size) {
+	for (; size > 0 && (c = getc_internal(gf)) != EOF; --size) {
 		if (c == '\n')
 			break;
 		*p++ = c;
@@ -1904,10 +2120,13 @@ gfs_pio_getline(GFS_File gf, char *s, size_t size, int *eofp)
 	*p++ = '\0';
 	if (p == s + 1 && c == EOF) {
 		*eofp = 1;
+		gfs_pio_mutex_unlock(&gf->mutex, __func__);
 		return (GFS_PIO_ERROR(gf));
 	}
 	*eofp = 0;
 
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+
 	gfs_profile(gfarm_gettimerval(&t2));
 	gfs_profile(staticp->getline_time += gfarm_timerval_sub(&t2, &t1));
 
@@ -1917,25 +2136,33 @@ gfs_pio_getline(GFS_File gf, char *s, size_t size, int *eofp)
 gfarm_error_t
 gfs_pio_putline(GFS_File gf, const char *s)
 {
-	gfarm_error_t e = gfs_pio_check_view_default(gf);
+	gfarm_error_t e;
 
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+
+	e = gfs_pio_check_view_default(gf);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1001333,
 			"gfs_pio_check_view_default() failed: %s",
 			gfarm_error_string(e));
+		gfs_pio_mutex_unlock(&gf->mutex, __func__);
 		return (e);
 	}
 
-	CHECK_WRITABLE(gf);
+	CHECK_WRITABLE_LOCKED(gf);
 
-	e = gfs_pio_puts(gf, s);
+	e = puts_internal(gf, s);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1001334,
 			"gfs_pio_puts() failed: %s",
 			gfarm_error_string(e));
+		gfs_pio_mutex_unlock(&gf->mutex, __func__);
 		return (e);
 	}
-	return (gfs_pio_putc(gf, '\n'));
+
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+
+	return (putc_internal(gf, '\n'));
 }
 
 #define ALLOC_SIZE_INIT	220
@@ -1949,7 +2176,7 @@ gfs_pio_putline(GFS_File gf, const char *s)
 gfarm_error_t
 gfs_pio_readline(GFS_File gf, char **bufp, size_t *sizep, size_t *lenp)
 {
-	gfarm_error_t e = gfs_pio_check_view_default(gf);
+	gfarm_error_t e;
 	char *buf = *bufp, *p = NULL;
 	size_t size = *sizep, len = 0;
 	int c;
@@ -1957,10 +2184,14 @@ gfs_pio_readline(GFS_File gf, char **bufp, size_t *sizep, size_t *lenp)
 	int overflow = 0;
 	gfarm_timerval_t t1, t2;
 
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+
+	e = gfs_pio_check_view_default(gf);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1001335,
 			"gfs_pio_check_view_default() failed: %s",
 			gfarm_error_string(e));
+		gfs_pio_mutex_unlock(&gf->mutex, __func__);
 		return (e);
 	}
 
@@ -1971,7 +2202,7 @@ gfs_pio_readline(GFS_File gf, char **bufp, size_t *sizep, size_t *lenp)
 #ifdef __GNUC__ /* workaround gcc warning: unused variable */
 	c = EOF;
 #endif
-	CHECK_READABLE(gf);
+	CHECK_READABLE_LOCKED(gf);
 
 	if (buf == NULL || size <= 1) {
 		if (size <= 1)
@@ -1981,11 +2212,12 @@ gfs_pio_readline(GFS_File gf, char **bufp, size_t *sizep, size_t *lenp)
 			gflog_debug(GFARM_MSG_1001336,
 				"allocation of buf for pio_getc failed: %s",
 				gfarm_error_string(GFARM_ERR_NO_MEMORY));
+			gfs_pio_mutex_unlock(&gf->mutex, __func__);
 			return (GFARM_ERR_NO_MEMORY);
 		}
 	}
 	for (;;) {
-		c = gfs_pio_getc(gf);
+		c = getc_internal(gf);
 		if (c == EOF)
 			break;
 		if (size <= len) {
@@ -2000,6 +2232,7 @@ gfs_pio_readline(GFS_File gf, char **bufp, size_t *sizep, size_t *lenp)
 					"failed or size overflow: %s",
 					gfarm_error_string(
 						GFARM_ERR_NO_MEMORY));
+				gfs_pio_mutex_unlock(&gf->mutex, __func__);
 				return (GFARM_ERR_NO_MEMORY);
 			}
 			buf = p;
@@ -2021,6 +2254,7 @@ gfs_pio_readline(GFS_File gf, char **bufp, size_t *sizep, size_t *lenp)
 				"or size overflow: %s",
 				gfarm_error_string(
 					GFARM_ERR_NO_MEMORY));
+			gfs_pio_mutex_unlock(&gf->mutex, __func__);
 			return (GFARM_ERR_NO_MEMORY);
 		}
 		buf = p;
@@ -2028,6 +2262,8 @@ gfs_pio_readline(GFS_File gf, char **bufp, size_t *sizep, size_t *lenp)
 	}
 	buf[len] = '\0';
 
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+
 	gfs_profile(gfarm_gettimerval(&t2));
 	/* XXX should introduce gfs_pio_readline_time??? */
 	gfs_profile(staticp->getline_time += gfarm_timerval_sub(&t2, &t1));
@@ -2049,17 +2285,21 @@ gfarm_error_t
 gfs_pio_readdelim(GFS_File gf, char **bufp, size_t *sizep, size_t *lenp,
 	const char *delim, size_t delimlen)
 {
-	gfarm_error_t e = gfs_pio_check_view_default(gf);
+	gfarm_error_t e;
 	char *buf = *bufp, *p = NULL;
 	size_t size = *sizep, len = 0, alloc_size;
 	int c, delimtail, overflow;
 	static const char empty_line[] = "\n\n";
 	gfarm_timerval_t t1, t2;
 
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+
+	e = gfs_pio_check_view_default(gf);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1001339,
 			"gfs_pio_check_view_default() failed: %s",
 			gfarm_error_string(e));
+		gfs_pio_mutex_unlock(&gf->mutex, __func__);
 		return (e);
 	}
 
@@ -2070,7 +2310,7 @@ gfs_pio_readdelim(GFS_File gf, char **bufp, size_t *sizep, size_t *lenp,
 #ifdef __GNUC__ /* workaround gcc warning: unused variable */
 	c = EOF;
 #endif
-	CHECK_READABLE(gf);
+	CHECK_READABLE_LOCKED(gf);
 
 	if (delim == NULL) { /* special case 1 */
 		delimtail = 0; /* workaround gcc warning */
@@ -2089,11 +2329,12 @@ gfs_pio_readdelim(GFS_File gf, char **bufp, size_t *sizep, size_t *lenp,
 			gflog_debug(GFARM_MSG_1001340,
 				"allocation of buf for pio_getc failed: %s",
 				gfarm_error_string(GFARM_ERR_NO_MEMORY));
+			gfs_pio_mutex_unlock(&gf->mutex, __func__);
 			return (GFARM_ERR_NO_MEMORY);
 		}
 	}
 	for (;;) {
-		c = gfs_pio_getc(gf);
+		c = getc_internal(gf);
 		if (c == EOF)
 			break;
 		if (size <= len) {
@@ -2108,6 +2349,7 @@ gfs_pio_readdelim(GFS_File gf, char **bufp, size_t *sizep, size_t *lenp,
 					" or size overflow: %s",
 					gfarm_error_string(
 						GFARM_ERR_NO_MEMORY));
+				gfs_pio_mutex_unlock(&gf->mutex, __func__);
 				return (GFARM_ERR_NO_MEMORY);
 			}
 			buf = p;
@@ -2120,11 +2362,11 @@ gfs_pio_readdelim(GFS_File gf, char **bufp, size_t *sizep, size_t *lenp,
 		    memcmp(&buf[len - delimlen], delim, delimlen) == 0) {
 			if (delim == empty_line) { /* special case 2 */
 				for (;;) {
-					c = gfs_pio_getc(gf);
+					c = getc_internal(gf);
 					if (c == EOF)
 						break;
 					if (c != '\n') {
-						gfs_pio_ungetc(gf, c);
+						ungetc_internal(gf, c);
 						break;
 					}
 					if (size <= len) {
@@ -2145,6 +2387,9 @@ gfs_pio_readdelim(GFS_File gf, char **bufp, size_t *sizep, size_t *lenp,
 							  gfarm_error_string(
 							    GFARM_ERR_NO_MEMORY
 								));
+							gfs_pio_mutex_unlock(
+							  &gf->mutex,
+							  __func__);
 							return (
 							  GFARM_ERR_NO_MEMORY);
 						}
@@ -2169,6 +2414,7 @@ gfs_pio_readdelim(GFS_File gf, char **bufp, size_t *sizep, size_t *lenp,
 				"or size overflow: %s",
 				gfarm_error_string(
 					GFARM_ERR_NO_MEMORY));
+			gfs_pio_mutex_unlock(&gf->mutex, __func__);
 			return (GFARM_ERR_NO_MEMORY);
 		}
 		buf = p;
@@ -2176,6 +2422,8 @@ gfs_pio_readdelim(GFS_File gf, char **bufp, size_t *sizep, size_t *lenp,
 	}
 	buf[len] = '\0';
 
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+
 	gfs_profile(gfarm_gettimerval(&t2));
 	/* XXX should introduce gfs_pio_readdelim_time??? */
 	gfs_profile(staticp->getline_time += gfarm_timerval_sub(&t2, &t1));
@@ -2207,11 +2455,15 @@ gfs_pio_view_fstat(GFS_File gf, struct gfs_stat *st)
 gfarm_error_t
 gfs_pio_stat(GFS_File gf, struct gfs_stat *st)
 {
-	gfarm_error_t e = gfs_pio_check_view_default(gf);
+	gfarm_error_t e;
 
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+
+	e = gfs_pio_check_view_default(gf);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1003742,
 		    "gfs_pio_stat: %s", gfarm_error_string(e));
+		gfs_pio_mutex_unlock(&gf->mutex, __func__);
 		return (e);
 	}
 	e = gfs_fstat(gf, st);
@@ -2219,6 +2471,7 @@ gfs_pio_stat(GFS_File gf, struct gfs_stat *st)
 		gflog_debug(GFARM_MSG_1001344,
 			"gfs_fstat() failed: %s",
 			gfarm_error_string(e));
+		gfs_pio_mutex_unlock(&gf->mutex, __func__);
 		return (e);
 	}
 
@@ -2227,7 +2480,8 @@ gfs_pio_stat(GFS_File gf, struct gfs_stat *st)
 			/* XXX call reconnect, when failover for writing
 			 *     is supported
 			 */
-			if ((e = gfs_pio_flush(gf)) != GFARM_ERR_NO_ERROR) {
+			if ((e = flush_internal(gf))
+			    != GFARM_ERR_NO_ERROR) {
 				gflog_debug(GFARM_MSG_1002655,
 				    "gfs_pio_flush() failed: %s",
 				    gfarm_error_string(e));
@@ -2256,24 +2510,32 @@ gfs_pio_stat(GFS_File gf, struct gfs_stat *st)
 			gf->error = e;
 		}
 	}
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
 	return (e);
 }
 
 gfarm_error_t
 gfs_pio_cksum(GFS_File gf, const char *type, struct gfs_stat_cksum *cksum)
 {
-	gfarm_error_t e = gfs_pio_check_view_default(gf);
+	gfarm_error_t e;
 	int nretries = GFS_FAILOVER_RETRY_COUNT;
 
+	gfs_pio_mutex_lock(&gf->mutex, __func__);
+
+	e = gfs_pio_check_view_default(gf);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1003743,
 		    "gfs_pio_cksum: %s", gfarm_error_string(e));
+		gfs_pio_mutex_unlock(&gf->mutex, __func__);
 		return (e);
 	}
 	do {
 		e = (*gf->ops->view_cksum)(gf, type, cksum);
 	} while (e != GFARM_ERR_NO_ERROR && --nretries >= 0 &&
 	    gfs_pio_failover_check_retry(gf, &e));
+
+	gfs_pio_mutex_unlock(&gf->mutex, __func__);
+
 	return (e);
 }
 
@@ -2294,15 +2556,18 @@ gfs_pio_recvfile(GFS_File r_gf, gfarm_off_t r_off,
 	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t1);
 	gfs_profile(gfarm_gettimerval(&t1));
 
+	gfs_pio_mutex_lock(&r_gf->mutex, __func__);
+
 	e = gfs_pio_check_view_default(r_gf);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1003741,
 		    "gfs_pio_check_view_default() failed: %s",
 		    gfarm_error_string(e));
+		gfs_pio_mutex_unlock(&r_gf->mutex, __func__);
 		return (e);
 	}
 
-	CHECK_READABLE(r_gf);
+	CHECK_READABLE_LOCKED(r_gf);
 
 	do {
 		e = (*r_gf->ops->view_recvfile)(r_gf, r_off, w_fd, w_off, len,
@@ -2313,6 +2578,7 @@ gfs_pio_recvfile(GFS_File r_gf, gfarm_off_t r_off,
 	gfs_profile(gfarm_gettimerval(&t2));
 	gfs_profile(staticp->read_time += gfarm_timerval_sub(&t2, &t1));
 
+	gfs_pio_mutex_unlock(&r_gf->mutex, __func__);
 	return (e);
 }
 
@@ -2329,15 +2595,18 @@ gfs_pio_sendfile(GFS_File w_gf, gfarm_off_t w_off,
 	GFARM_TIMEVAL_FIX_INITIALIZE_WARNING(t1);
 	gfs_profile(gfarm_gettimerval(&t1));
 
+	gfs_pio_mutex_lock(&w_gf->mutex, __func__);
+
 	e = gfs_pio_check_view_default(w_gf);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1003952,
 			"gfs_pio_check_view_default() failed: %s",
 			gfarm_error_string(e));
+		gfs_pio_mutex_unlock(&w_gf->mutex, __func__);
 		return (e);
 	}
 
-	CHECK_WRITABLE(w_gf);
+	CHECK_WRITABLE_LOCKED(w_gf);
 
 	do {
 		e = (*w_gf->ops->view_sendfile)(w_gf, w_off, r_fd, r_off, len,
@@ -2348,6 +2617,8 @@ gfs_pio_sendfile(GFS_File w_gf, gfarm_off_t w_off,
 	gfs_profile(gfarm_gettimerval(&t2));
 	gfs_profile(staticp->write_time += gfarm_timerval_sub(&t2, &t1));
 
+	gfs_pio_mutex_unlock(&w_gf->mutex, __func__);
+
 	return (e);
 }
 
