@@ -293,31 +293,27 @@ gfarm_hostspec_match(struct gfarm_hostspec *hostspecp,
 	return (0);
 }
 
-void
+/* string can be NULL */
+int
 gfarm_hostspec_to_string(struct gfarm_hostspec *hostspec,
 	char *string, size_t size)
 {
 	unsigned char *a, *m;
 
-	if (size <= 0)
-		return;
-
 	switch (hostspec->type) {
 	case GFHS_ANY:
-		string[0] = '\0';
-		return;
+		return (snprintf(string, size, "*"));
 	case GFHS_NAME:
-		strncpy(string, hostspec->u.name, size);
-		return;
+		return (snprintf(string, size, "%s", hostspec->u.name));
 	case GFHS_AF_INET4:
 		a = (unsigned char *)&hostspec->u.in4_addr.addr.s_addr;
 		m = (unsigned char *)&hostspec->u.in4_addr.mask.s_addr;
-		snprintf(string, size, "%d.%d.%d.%d/%d.%d.%d.%d",
-		    a[0], a[1], a[2], a[3], m[0], m[1], m[2], m[3]);
-		return;
+		return (snprintf(string, size, "%d.%d.%d.%d/%d.%d.%d.%d",
+		    a[0], a[1], a[2], a[3], m[0], m[1], m[2], m[3]));
 	}
 	/* assert(0); */
-	return;
+	return (snprintf(string, size,
+	    "INTERNAL-ERROR: gfarm_hostspec_to_string: invalid type"));
 }
 
 #ifndef __KERNEL__	/* gfarm_sockaddr_to_name:: apl */
