@@ -16,5 +16,7 @@ int gfarm_iobuffer_blocking_read_timeout_fd_op(struct gfarm_iobuffer *,
 	void *, int, void *, int);
 int gfarm_iobuffer_blocking_read_notimeout_fd_op(struct gfarm_iobuffer *,
 	void *, int, void *, int);
-int gfarm_iobuffer_blocking_write_socket_op(struct gfarm_iobuffer *,
+int gfarm_iobuffer_blocking_write_timeout_socket_op(struct gfarm_iobuffer *,
+	void *, int, void *, int);
+int gfarm_iobuffer_blocking_write_notimeout_socket_op(struct gfarm_iobuffer *,
 	void *, int, void *, int);
