@@ -4,6 +4,7 @@ local_dir1=$localtmp/dir1
 local_dir2=$localtmp/dir2
 local_tmpfile1=$localtmp/tmpfile1
 local_tmpfile2=$localtmp/tmpfile2
+local_tmpfile3=$localtmp/tmpfile3
 
 GFPREP=$regress/bin/gfprep_for_test
 GFPCOPY=$regress/bin/gfpcopy_for_test
@@ -42,20 +43,21 @@ setup_test() {
 check_N() {
   P=$1
   N=$2
-  if gfstat $P > $local_tmpfile2; then
+  local_tmpfile_gfstat=$localtmp/tmpfile_gfstat
+  if gfstat $P > $local_tmpfile_gfstat; then
     :
   else
     echo failed gfstat
-    cat $local_tmpfile2
+    cat $local_tmpfile_gfstat
     clean_test
     exit $exit_fail
   fi
-  if [ `awk '/Ncopy/{print $NF}' $local_tmpfile2` -eq $N ]; then
+  if [ `awk '/Ncopy/{print $NF}' $local_tmpfile_gfstat` -eq $N ]; then
     :
   else
     echo "unexpected number of replicas (not $N)"
     gfwhere -al $P
-    cat $local_tmpfile2
+    cat $local_tmpfile_gfstat
     clean_test
     exit $exit_fail
   fi
