@@ -113,20 +113,24 @@ sync_protocol_get_thrpool(void)
 gfarm_error_t
 gfm_server_protocol_extension_default(struct peer *peer,
 	int from_client, int skip, int level, gfarm_int32_t request,
+	gfarm_int32_t last_request,
 	gfarm_int32_t *requestp, gfarm_error_t *on_errorp)
 {
 	gflog_warning(GFARM_MSG_1000181, "unknown request: %d", request);
+	gflog_info(GFARM_MSG_1005215, "last request: %d", last_request);
 	peer_record_protocol_error(peer);
 	return (GFARM_ERR_PROTOCOL);
 }
 
 /* this interface is made as a hook for a private extension */
 gfarm_error_t (*gfm_server_protocol_extension)(struct peer *,
-	int, int, int, gfarm_int32_t, gfarm_int32_t *, gfarm_error_t *) =
+	int, int, int, gfarm_int32_t, gfarm_int32_t,
+	gfarm_int32_t *, gfarm_error_t *) =
 		gfm_server_protocol_extension_default;
 
 gfarm_error_t
 protocol_switch(struct peer *peer, int from_client, int skip, int level,
+	gfarm_int32_t last_request,
 	gfarm_int32_t *requestp, gfarm_error_t *on_errorp, int *suspendedp)
 {
 	gfarm_error_t e, e2;
@@ -639,7 +643,8 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 		break;
 	default:
 		e = gfm_server_protocol_extension(peer,
-		    from_client, skip, level, request, requestp, on_errorp);
+		    from_client, skip, level, request, last_request,
+		    requestp, on_errorp);
 		break;
 	}
 
@@ -675,6 +680,7 @@ void
 protocol_state_init(struct protocol_state *ps)
 {
 	ps->nesting_level = 0;
+	ps->last_request = -1;
 }
 
 /*
@@ -703,8 +709,9 @@ protocol_service(struct peer *peer)
 
 	from_client = peer_get_auth_id_type(peer) == GFARM_AUTH_ID_TYPE_USER;
 	if (ps->nesting_level == 0) { /* top level */
-		e = protocol_switch(peer, from_client, 0, 0,
+		e = protocol_switch(peer, from_client, 0, 0, ps->last_request,
 		    &request, &dummy, &suspended);
+		ps->last_request = request;
 		if (suspended)
 			return (1); /* finish */
 		giant_lock();
@@ -727,7 +734,8 @@ protocol_service(struct peer *peer)
 		giant_unlock();
 	} else { /* inside of a COMPOUND block */
 		e = protocol_switch(peer, from_client, cs->skip, 1,
-		    &request, &cs->current_part, &suspended);
+		    ps->last_request, &request, &cs->current_part, &suspended);
+		ps->last_request = request;
 		if (suspended)
 			return (1); /* finish */
 		if (peer_had_protocol_error(peer)) {
@@ -1243,26 +1251,26 @@ dynamic_config_include_file(char *rest_of_line, const char *file, int lineno)
 	p = rest_of_line;
 	e = gfarm_config_strtoken(&p, &s);
 	if (e != GFARM_ERR_NO_ERROR) {
-		gflog_error(GFARM_MSG_UNFIXED,
+		gflog_error(GFARM_MSG_1005216,
 		    "file %s, line %d: parsing token (%s) failed: %s",
 		    file, lineno, p, gfarm_error_string(e));
 		return;
 	}
 	if (s == NULL) {
-		gflog_error(GFARM_MSG_UNFIXED,
+		gflog_error(GFARM_MSG_1005217,
 		    "file %s, line %d: missing include filename",
 		    file, lineno);
 		return;
 	}
 	e = gfarm_config_strtoken(&p, &tmp);
 	if (e != GFARM_ERR_NO_ERROR) {
-		gflog_error(GFARM_MSG_UNFIXED,
+		gflog_error(GFARM_MSG_1005218,
 		    "file %s, line %d: parsing token (%s) failed: %s",
 		    file, lineno, p, gfarm_error_string(e));
 		return;
 	}
 	if (tmp != NULL) {
-		gflog_error(GFARM_MSG_UNFIXED,
+		gflog_error(GFARM_MSG_1005219,
 		    "file %s, line %d: too many arguments for include",
 		    file, lineno);
 		return;
@@ -1270,7 +1278,7 @@ dynamic_config_include_file(char *rest_of_line, const char *file, int lineno)
 
 	++gfarm_ctxp->include_nesting_level;
 	if (gfarm_ctxp->include_nesting_level > nesting_limit) {
-		gflog_error(GFARM_MSG_UNFIXED,
+		gflog_error(GFARM_MSG_1005220,
 		    "file %s, line %d: include nesting level %d "
 		    "exceeds include_nesting_limit %d",
 		    file, lineno,
@@ -1282,7 +1290,7 @@ dynamic_config_include_file(char *rest_of_line, const char *file, int lineno)
 	if (s[0] != '/')  {
 		malloced_filename = gfarm_config_dirname_add(s, file);
 		if (malloced_filename == NULL) {
-			gflog_error(GFARM_MSG_UNFIXED,
+			gflog_error(GFARM_MSG_1005221,
 			    "file %s, line %d: no memory to include %s",
 			    file, lineno, s);
 		}
@@ -1290,7 +1298,7 @@ dynamic_config_include_file(char *rest_of_line, const char *file, int lineno)
 	}
 	config = fopen(s, "r");
 	if (config == NULL) {
-		gflog_notice(GFARM_MSG_UNFIXED,
+		gflog_notice(GFARM_MSG_1005222,
 		    "file %s, line %d: %s: cannot open include file",
 		    file, lineno, s);
 		free(malloced_filename);
@@ -1319,7 +1327,7 @@ dynamic_config_read(FILE *config, const char *file)
 
 		e = gfarm_config_strtoken(&p, &s);
 		if (e != GFARM_ERR_NO_ERROR) {
-			gflog_error(GFARM_MSG_UNFIXED,
+			gflog_error(GFARM_MSG_1005223,
 			    "%s, line %d: invalid token: %s",
 			    file, lineno, gfarm_error_string(e));
 			/* don't stop, just ignore this line line */
@@ -1346,7 +1354,7 @@ dynamic_config_read_file(const char *file)
 	FILE *config = fopen(file, "r");
 
 	if (config == NULL) {
-		gflog_notice(GFARM_MSG_UNFIXED,
+		gflog_notice(GFARM_MSG_1005224,
 		    "%s: cannot open config file", file);
 		return;
 	}
