@@ -22,9 +22,11 @@ extern struct thread_pool *authentication_thread_pool;
 struct thread_pool *sync_protocol_get_thrpool(void);
 
 gfarm_error_t gfm_server_protocol_extension_default(struct peer *,
-	int, int, int, gfarm_int32_t, gfarm_int32_t *, gfarm_error_t *);
+	int, int, int, gfarm_int32_t, gfarm_int32_t,
+	gfarm_int32_t *, gfarm_error_t *);
 extern gfarm_error_t (*gfm_server_protocol_extension)(struct peer *,
-	int, int, int, gfarm_int32_t, gfarm_int32_t *, gfarm_error_t *);
+	int, int, int, gfarm_int32_t, gfarm_int32_t,
+	gfarm_int32_t *, gfarm_error_t *);
 
 int protocol_service(struct peer *);
 void *protocol_main(void *);
