@@ -203,6 +203,9 @@ group_remove_internal(const char *groupname, int update_quota)
 	}
 	if (update_quota)
 		quota_group_remove(g);
+
+	/* NOTE: this group remains in struct inode */
+
 	/*
 	 * do not purge the hash entry.  Instead, invalidate it so
 	 * that it can be activated later.
@@ -269,6 +272,12 @@ group_name(struct group *g)
 	    g->groupname : REMOVED_GROUP_NAME);
 }
 
+char *
+group_name_even_invalid(struct group *g)
+{
+	return (g != NULL ? g->groupname : REMOVED_GROUP_NAME);
+}
+
 char *
 group_name_with_invalid(struct group *g)
 {
@@ -289,10 +298,11 @@ group_usage_tmp(struct group *g)
 
 void
 group_foreach(void *closure, void (*callback)(void *, struct group *),
-	int valid_only)
+	int flags)
 {
 	struct gfarm_hash_iterator it;
 	struct group **g;
+	int valid_only = (flags & GROUP_FOREARCH_FLAG_VALID_ONLY) != 0;
 
 	for (gfarm_hash_iterator_begin(group_hashtab, &it);
 	     !gfarm_hash_iterator_is_end(&it);
@@ -801,6 +811,10 @@ gfm_server_group_info_set(struct peer *peer, int from_client, int skip)
 		gflog_debug(GFARM_MSG_1001537,
 			"group_user_check() failed: %s",
 			gfarm_error_string(e));
+	} else if (gfarm_read_only_mode()) {
+		gflog_debug(GFARM_MSG_1005145, "%s (%s@%s) during read_only",
+		    diag, peer_get_username(peer), peer_get_hostname(peer));
+		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	/*
 	 * We have to call this before group_info_add(),
 	 * because group_info_add() frees the memory of gi
@@ -887,10 +901,14 @@ gfm_server_group_info_modify(struct peer *peer, int from_client, int skip)
 		gflog_debug(GFARM_MSG_1001541,
 			"group_lookup() failed");
 		e = GFARM_ERR_NO_SUCH_GROUP;
-	} else if ((e = group_user_check(&gi, diag)) != GFARM_ERR_NO_ERROR)
-		gflog_debug(GFARM_MSG_1001542,
-			"group_user_check() failed: %s", gfarm_error_string(e));
-	else {
+	} else if ((e = group_user_check(&gi, diag)) != GFARM_ERR_NO_ERROR) {
+		gflog_debug(GFARM_MSG_1001542, "group_user_check() failed: %s",
+		    gfarm_error_string(e));
+	} else if (gfarm_read_only_mode()) {
+		gflog_debug(GFARM_MSG_1005146, "%s (%s@%s) during read_only",
+		    diag, peer_get_username(peer), peer_get_hostname(peer));
+		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
+	} else {
 		group_modify(group, &gi, diag);
 		/* change all entries */
 		e = db_group_modify(&gi, 0, 0, NULL, 0, NULL);
@@ -954,6 +972,10 @@ gfm_server_group_info_remove(struct peer *peer, int from_client, int skip)
 		    "%s: administrator group \"%s\" should not be deleted",
 		    diag, groupname);
 		e = GFARM_ERR_OPERATION_NOT_PERMITTED;
+	} else if (gfarm_read_only_mode()) {
+		gflog_debug(GFARM_MSG_1005147, "%s (%s@%s) during read_only",
+		    diag, peer_get_username(peer), peer_get_hostname(peer));
+		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	} else
 		e = group_info_remove(groupname, diag);
 	free(groupname);
