@@ -1888,7 +1888,7 @@ gfm_server_host_info_set(struct peer *peer, int from_client, int skip)
 	} else if ((e = host_info_verify(&hi, diag)) != GFARM_ERR_NO_ERROR) {
 		/* nothing to do */
 	} else if (gfarm_read_only_mode()) {
-		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) during read_only",
+		gflog_debug(GFARM_MSG_1005154, "%s (%s@%s) during read_only",
 		    diag, peer_get_username(peer), peer_get_hostname(peer));
 		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	} else if ((e = host_enter(&hi, NULL)) != GFARM_ERR_NO_ERROR) {
@@ -1997,7 +1997,7 @@ gfm_server_host_info_modify(struct peer *peer, int from_client, int skip)
 	} else if ((e = host_info_verify(&hi, diag)) != GFARM_ERR_NO_ERROR) {
 		needs_free = 1;
 	} else if (gfarm_read_only_mode()) {
-		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) during read_only",
+		gflog_debug(GFARM_MSG_1005155, "%s (%s@%s) during read_only",
 		    diag, peer_get_username(peer), peer_get_hostname(peer));
 		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 		needs_free = 1;
@@ -2103,7 +2103,7 @@ gfm_server_host_info_remove(struct peer *peer, int from_client, int skip)
 			"operation is not permitted");
 		e = GFARM_ERR_OPERATION_NOT_PERMITTED;
 	} else if (gfarm_read_only_mode()) {
-		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) during read_only",
+		gflog_debug(GFARM_MSG_1005156, "%s (%s@%s) during read_only",
 		    diag, peer_get_username(peer), peer_get_hostname(peer));
 		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	} else
