@@ -927,7 +927,7 @@ gfm_server_user_info_set(struct peer *peer, int from_client, int skip)
 	} else if ((e = user_info_verify(&ui, diag)) != GFARM_ERR_NO_ERROR) {
 		/* nothing to do */
 	} else if (gfarm_read_only_mode()) {
-		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) during read_only",
+		gflog_debug(GFARM_MSG_1005142, "%s (%s@%s) during read_only",
 		    diag, peer_get_username(peer), peer_get_hostname(peer));
 		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	} else {
@@ -1038,7 +1038,7 @@ gfm_server_user_info_modify(struct peer *peer, int from_client, int skip)
 		gflog_debug(GFARM_MSG_1003462,
 		    "%s: user_info_verify: %s", diag, gfarm_error_string(e));
 	} else if (gfarm_read_only_mode()) {
-		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) during read_only",
+		gflog_debug(GFARM_MSG_1005143, "%s (%s@%s) during read_only",
 		    diag, peer_get_username(peer), peer_get_hostname(peer));
 		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	} else if ((e = user_modify(u, &ui)) != GFARM_ERR_NO_ERROR) {
@@ -1107,7 +1107,7 @@ gfm_server_user_info_remove(struct peer *peer, int from_client, int skip)
 			"operation is not permitted");
 		e = GFARM_ERR_OPERATION_NOT_PERMITTED;
 	} else if (gfarm_read_only_mode()) {
-		gflog_debug(GFARM_MSG_UNFIXED, "%s (%s@%s) during read_only",
+		gflog_debug(GFARM_MSG_1005144, "%s (%s@%s) during read_only",
 		    diag, peer_get_username(peer), peer_get_hostname(peer));
 		e = GFARM_ERR_READ_ONLY_FILE_SYSTEM;
 	} else
